
/******************************************************************************
  DAYTIME - read and print time of day string from internet

  Copyright (C) 1991, University of Waterloo
  Portions Copyright (C) 1990, National Center for Supercomputer Applications

  This program is free software; you can redistribute it and/or modify
  it, but you may not sell it.

  This program is distributed in the hope that it will be useful,
  but without any warranty; without even the implied warranty of
  merchantability or fitness for a particular purpose.

      Erick Engelke                   or via E-Mail
      Faculty of Engineering
      University of Waterloo          Erick@development.watstar.uwaterloo.ca
      200 University Ave.,
      Waterloo, Ont., Canada
      N2L 3G1

  Returns:
      0   - success

      2   - some failure in the connection (port unavailable,
              no response, etc.)
      3   - unable to reach it - local host or first router is down

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tcp.h>

#define DAYTIME_PORT 13

#ifndef __SMALL__
#define TCP_DAYTIME
#endif

int daytime (DWORD host)
{
  static tcp_Socket sock;
  tcp_Socket *s = &sock;
  char buffer [513];
  int  retcode = 3;
  int  status  = 0;

#ifdef TCP_DAYTIME
  if (!tcp_open(s,0,host,DAYTIME_PORT,NULL))
  {
    outs ("Sorry, unable to connect to that machine right now!\n");
    return (3);
  }
  outs ("waiting...\r");
  sock_wait_established (s,sock_delay,NULL,&status);
  outsnl ("connected ");

#else
  int  udpretries = 3;
  long udpretrytime;

  udp_open (s,0,host,DAYTIME_PORT,NULL);
  sock_write (s,"\n",1);
  udpretrytime = set_timeout (2000);
#endif

  while (1)
  {
    sock_tick (s,&status);

#ifndef TCP_DAYTIME
    if (chk_timeout(udpretrytime))
    {
      if (udpretries-- == 0)
         break;
      udpretrytime = set_timeout (2000);
      sock_write(s,"\n",1);
    }
#endif
    if (sock_dataready(s))
    {
      sock_gets (s,buffer,sizeof(buffer));
      outsnl (buffer);
      retcode = 0;
      break;
    }
  }
  sock_close (s);
  sock_wait_closed (s,sock_delay,NULL,&status);

sock_err:
  switch (status)
  {
    case 1 : /* foreign host closed */
             return (retcode);
    case -1: /* timeout/abort       */
             outsnl (sockerr(s));
             return (2);
    default: outs ("\nAborting\n");
             return (2);
  }
}


int main (int argc, char **argv)
{
  DWORD host;

  if (argc < 2)
  {
    outsnl("DAYTIME server [-d]");
    exit(3);
  }

  if (argc == 3 && !strcmp(argv[2],"-d"))
  {
    tcp_set_debug_state(2);
    dbug_init();
  }

  sock_init();

  if ((host = lookup_host(argv[1],NULL)) == 0L)
  {
    outs ("Could not resolve host `");
    outs (argv[1]);
    outs ("'\n");
    return (3);
  }
  
  return daytime (host);;
}
