#
# djcommon.mak: make rules for Watt-32 binaries (djgpp2)
#
# Note: This file requires GNU Make 3.75 or later
#       This file requires a non-Unix shell.
#
# by Gisle Vanem 1997
#

.SUFFIXES: .l .y .exe
.PHONY:    check_src check_exe

#
# Override any Unix-like SHELL set in environment or djgpp.env
# At-least "make depend" needs a DOS-style shell. Specifically
# the "sed" command doesn't work with `SHELL=sh' or `bash'.
#
# MAKESHELL = command.com  # doesn't have the wanted effect
# SHELL = $(COMSPEC)       # doesn't work

ifeq ($(SRC),)
check_src:
	@echo \"SRC\" not defined. Define \"SRC = C-files\".. before including DJCOMMON.MAK
endif

ifeq ($(DJGPP_EXE),)
check_exe:
	@echo \"DJGPP_EXE\" not defined. Define \"DJGPP_EXE = file.exe\".. before including DJCOMMON.MAK
endif

OBJS += $(SRC:.c=.o)
CC    = gcc.exe

LFLAGS = ../../lib/libwatt.a
CFLAGS = -I../../inc -g -Wall -O2 -m486 -D__WATT32__

#
# Define 'STRIP_EXE = 1' if you like a smaller .exe-file (DJGPP_EXE)
# Define 'PACK_EXE = 1' to also compress using UPX .exe-packer
#
ifeq ($(STRIP_EXE),1)
  LFLAGS += -s
endif

ifeq ($(PACK_EXE),1)
  LFLAGS += -s
endif

#
# Define 'MAKE_MAP = 1' if you like a .map-file
#
ifeq ($(MAKE_MAP),1)
  LINK = redir -e $(DJGPP_EXE:.exe=.map) $(CC) -Wl,--print-map,--sort-common,--cref
else
  LINK = $(CC)
endif

all: $(DJGPP_EXE)

$(DJGPP_EXE): $(OBJS)
	$(LINK) $^ -o $@ $(strip $(LFLAGS))
ifeq ($(PACK_EXE),1)
	@upx $@
endif

%.o: %.c
	$(CC) -c $(strip $(CFLAGS)) -o $@ $<

clean:
	rm -f $(OBJS) Makefile.bak $(DJGPP_EXE:.exe=.map)

vclean scrub:  clean
	rm -f $(DJGPP_EXE) $(GENERATED)

#
# The following hack is to bypass a small bug in gcc 2.95.x;
# It includes some system headers in dependency output.
#
MM_SPECS = $(subst \,/,$(TMPDIR))/specs

$(MM_SPECS)::
	@echo "*cpp: %(cpp_cpu) %{posix:-D_POSIX_SOURCE} -remap" > $@

depend: $(MM_SPECS)
	@echo Generating dependencies..
	- copy Makefile.dj Makefile.bak
	sed -e "/^# DO NOT DELETE THIS LINE/,$$d" < Makefile.bak > Makefile.dj
	echo "# DO NOT DELETE THIS LINE"                        >> Makefile.dj
	$(CC) -specs=$(MM_SPECS) -MM $(CFLAGS) $(SRC)           >> Makefile.dj
	rm -f $(MM_SPECS)
