/******************************************************************************

  TCPINFO - display configuration info to the screen

  Copyright (C) 1991, University of Waterloo
  portions Copyright (C) 1990, National Center for Supercomputer Applications

  This program is free software; you can redistribute it and/or modify
  it, but you may not sell it.

  This program is distributed in the hope that it will be useful,
  but without any warranty; without even the implied warranty of
  merchantability or fitness for a particular purpose.

      Erick Engelke                   or via E-Mail
      Faculty of Engineering
      University of Waterloo          Erick@development.watstar.uwaterloo.ca
      200 University Ave.,
      Waterloo, Ont., Canada
      N2L 3G1

******************************************************************************/

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <tcp.h>

int unused = 0;

void my_hook (const char *name, const char *value)
{
  unused++;
#if 0
  printf ("   unknown: %s = %s\n", name, value);
#else
 name  = name;  /* argused */
 value = value;
#endif
}

/* private/undocumented variables
 */
extern eth_address  _eth_addr[];
extern WORD  _pktdevclass;
extern DWORD _bootphost;
extern WORD  _bootptimeout;
extern WORD  _bootpon;

#define MAX_ARP_DATA  40  /* max # of entries in ARP-cache */

char buffer [512], buf2 [512];

char *EtherAddr (void *eth_adr)
{
  static char buf[20];
  char  *eth = (char*)eth_adr;

  sprintf (buf, "%02X:%02X:%02X:%02X:%02X:%02X",
           eth[0] & 255, eth[1] & 255, eth[2] & 255,
           eth[3] & 255, eth[4] & 255, eth[5] & 255);
  return (buf);
}

int main (void)
{
  int  i, num_cached = 0;
  char hostname[50];

  puts ("Reading Waterloo TCP configuration file.");

  usr_init = my_hook;
  _survivebootp = 1;  /* needed to not exit if BOOTP fails */
  _survivedhcp  = 1;  /* ditto for DHCP/RARP */
  _surviverarp  = 1;

  sock_init();

  if (unused)
     printf ("\n%d extra parameters were found in your configuration file.\n"
             "These values are extensions used by applications or BSD-sockets.\n",
             unused);

  switch (_pktdevclass)
  {
    case 3:
         printf ("\nToken Ring not supported\n");
         break;
    case 1:
         printf ("\nEthernet Address : %s\n", EtherAddr(&_eth_addr));
         break;
    case 6:
         puts ("Protocol         : SLIP");
         break;
    case 18:
         puts ("Protocol         : PPP");
         break;
  }

  if (multihomes)
       printf ("\nIP Addresses     : %s - %s\n",
               _inet_ntoa (buf2,  _gethostid()),
               _inet_ntoa (buffer,_gethostid()+multihomes));
  else printf ("\nIP Address       : %s\n", _inet_ntoa (buffer,_gethostid()));

  printf ("Network Mask     : %s\n\n", _inet_ntoa(buffer,sin_mask));
  printf ("Gateways         : ");

  if (!arp_last_gateway)
       puts ("NONE");
  else puts ("GATEWAY'S IP     SUBNET           SUBNET MASK");

  for (i = 0 ; i < arp_last_gateway; i++)
  {
    char gate[20], snet[20], mask[20];

    strcpy (gate, _inet_ntoa(gate,arp_gate_list[i].gate_ip));
    strcpy (snet, _inet_ntoa(snet,arp_gate_list[i].subnet));
    strcpy (mask, _inet_ntoa(mask,arp_gate_list[i].mask));

    printf ("                 : %-15s  %-15s  %-15s\n",
            gate, arp_gate_list[i].subnet ? snet : "DEFAULT", mask);
  }

  puts ("\nARP Cache        : IP Address       Ether Address");
  for (i = 0; i < MAX_ARP_DATA; i++)
  {
    if (arp_cache_data[i].flags >= 1)
    {
      num_cached++;
      printf ("                 : %-15s  %s\n",
               _inet_ntoa(buffer, arp_cache_data[i].ip),
              EtherAddr (&arp_cache_data[i].hardware));
    }
  }
  if (num_cached == 0)
       puts ("                   <None>           <None>\n");
  else puts ("");

  if (gethostname(hostname,sizeof(hostname)) == 0)
     printf ("Host name        : %s\n", hostname);

  if (def_domain && *def_domain)
       printf ("Domain name      : %s\n", def_domain);
  else printf ("Domain name      : <none>\n");

  printf ("Cookieserver%c    : ", (last_cookie < 2) ? ' ' : 's');
  if (!last_cookie)
     puts ("NONE DEFINED");

  for (i = 0 ; i < last_cookie; i++)
  {
    if (i)
       printf ("                 : ");
    printf ("%s\n", _inet_ntoa(buffer, cookies[i]));
  }
  puts("");

  printf ("Nameserver%c      : ", (last_nameserver < 2) ? ' ' : 's');
  if (!last_nameserver)
     puts ("NONE DEFINED\n");

  for (i = 0 ; i < last_nameserver; i++)
  {
    if (i) printf ("                 : ");
    printf ("%s\n", _inet_ntoa(buffer, def_nameservers[i]));
  }

  if (_bootpon)
  {
    printf ("BOOTP            : Enabled and %s\n", _gethostid() ? "SUCCEEDED" : "FAILED");
    printf ("BOOTP Server     : %s\n",           _inet_ntoa(buffer,_bootphost));
    printf ("BOOTP Timeout    : %i seconds\n\n", _bootptimeout);
  }

  printf ("Default Timeout  : %u seconds\n", sock_delay);
  printf ("Max Seg Size,MSS : %u bytes\n",   mss);
  printf ("Max Transmit,MTU : %u bytes\n\n", mtu);

  printf ("tcp_Socket size  : %d bytes\n",   (int)sizeof(tcp_Socket));
  printf ("udp_Socket size  : %d bytes\n\n", (int)sizeof(udp_Socket));

  printf ("Version info     : %s\n", wattcpVersion());
  printf ("Capabilities     : %s\n", wattcpCapabilities());
  return (0);
}
